#ifndef TEMP_MINIZ_HXX
#define TEMP_MINIZ_HXX

#include <vector>
#include <string>
#include <boost/asio/awaitable.hpp>

typedef unsigned char mz_uint8;
typedef signed short mz_int16;
typedef unsigned short mz_uint16;
typedef unsigned int mz_uint32;
typedef unsigned int mz_uint;
typedef uint64_t mz_uint64;
typedef bool mz_bool;

struct ZipArchive {
    boost::asio::awaitable<void> add_dir(const std::string& path);
    boost::asio::awaitable<void> add_file(const std::string& file_path, const std::string& archive_path);
    boost::asio::awaitable<void> end();

protected:
    virtual boost::asio::awaitable<void> write(const void *, size_t n) = 0;

private:
    mz_uint64 m_archive_size = 0;
    mz_uint32 m_total_files = 0;

    std::vector<mz_uint8> m_central_dir;
    std::vector<mz_uint32> m_central_dir_offsets;

    void add_to_central_dir(const char *pFilename, mz_uint16 filename_size,
                                                const char *pExtra, mz_uint16 extra_size,
                                                mz_uint64 uncomp_size, mz_uint64 comp_size, mz_uint32 uncomp_crc32,
                                                mz_uint16 bit_flags,
                                                mz_uint64 local_header_ofs, mz_uint32 ext_attributes);
};


#endif //TEMP_MINIZ_HXX
